#include <LiquidCrystal_I2C.h>;
#include "dht11.h"

dht11 DHT11;
LiquidCrystal_I2C lcd(0x27, 16, 2);

#define DHT11PIN 4

uint8_t state = 0;
uint8_t print_state = -1;
float temperature_result;
float humidity_result = 0;
unsigned long button1_press_t = 0;
unsigned long button2_press_t = 0;
uint16_t time_selected = 0;
uint8_t time_printed_state = -1;

int time_array[5] = {8, 10, 12, 15, 20};
uint16_t time_index = 0;

uint16_t measuring = 0;
int time_step = 0;

float color_intensity;

void configure_timer() {

  // setare CTC
  TCCR0A |= (1 << WGM01);

  // setare TOP => frecventa 1kHz
  OCR0A = 250;

  // setare prescaler 64
  TCCR0A |= (1 << CS01); TCCR0A |= (1 << CS00);

  // activeaza intreruperea pe compare
  TIMSK0 |= (1 << OCIE0A);
}

void setup_pwm_led() {
  // activeaza output pentru pin led
  DDRD |= (1 << PD3);
  DDRB |= (1 << PB3);

  // activeaza CTC a timerului 2
  TCCR2A |= (1 << COM2B1);
  TCCR2A |= (1 << COM2A1);

  // activeaza fast PWM
  TCCR2A |= (1 << WGM21);
  TCCR2A |= (1 << WGM20);

  // prescaler 256
  TCCR2B |= (1 << CS22);

  OCR2A = color_intensity;
  OCR2B = 255 - color_intensity;
}

ISR(TIMER0_COMPA_vect) {
  // cod întrerupere

  if(measuring == 1) {
    if(time_step > 0) {
      time_step = time_step - 1;
    } else {
      measuring = 0;
      state = 3;
    }

    color_intensity = (((float)(time_array[time_index] * 1000 - time_step)) / ((float)(time_array[time_index] * 1000))) * 255;
  }
}

ISR(PCINT2_vect) {

  // BUTTON 2
  if ((PIND & (1 << PD2)) == 0) {
    if(millis() - button2_press_t > 200) {
      if(state == 1) {
        time_selected = 1;
        time_index++;
        if(time_index >= 5)
          time_index = 0;
      }

      if(state == 2) {
        state = 3;
      }
      button2_press_t = millis();
    }
  }

  // BUTTON 1
  if ((PIND & (1 << PD7)) == 0) {
    if(millis() - button1_press_t > 200) {

      if(state == 0) {
        state = 1;
        time_selected = 0;
      }

      if(state == 1 && time_selected == 1) {
        state = 2;
      }

      if(state == 3) {
        state = 0;
      }
      button1_press_t = millis();
    }
  }
}

/*
ISR(PCINT0_vect) {

  if(millis() - button2_press_t > 200) {
    if(state == 1) {
      time_selected = 1;
      time_index++;
      if(time_index >= 5)
        time_index = 0;

      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Selected time:");
      lcd.setCursor(0, 1);
      lcd.print(time_array[time_index]);
      Serial.print("Selected time: ");
      Serial.println(time_array[time_index]);
    }

    if(state == 2) {
      state = 3;
    }
    button2_press_t = millis();
  }
}*/

void setup() {
  // put your setup code here, to run once:
  lcd.begin();
  state = 0;
  temperature_result = 0;
  humidity_result = 0;
  cli();

  // input
  DDRD &= ~(1 << PD7);
  DDRD &= ~(1 << PD2);
  // input pullup
  PORTD |= (1 << PD7);
  PORTD |= (1 << PD2);

  // setup BUTTON1
  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT23);

  // setul BUTTON2
  //PCICR |= (1 << PCIE0);
  PCMSK2 |= (1 << PCINT18);

  // configure timer
  configure_timer();

  sei();
  lcd.clear();
  lcd.setCursor(0, 0);
  Serial.begin(9600);
}

void loop() {

  if(state == 0) {
    humidity_result = 0;
    temperature_result = 0;
    measuring = 0;
    time_printed_state = -1;
    time_selected = 0;
    
    // print start
    if(print_state != 0) {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Press B1");
      lcd.setCursor(0, 1);
      lcd.print("to start!");
      print_state = 0;
      Serial.println("Press B1 to start!");
    }  
  }

  if(state == 1) {
    if(print_state != 1) {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Press B2 to ");
      lcd.setCursor(0, 1);
      lcd.print("choose time");
      print_state = 1;
      Serial.println("Press B2 to choose time");
    }

    if(time_selected == 1 && time_printed_state != time_index) {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Selected time:");
      lcd.setCursor(0, 1);
      lcd.print(time_array[time_index]);
      Serial.print("Selected time: ");
      Serial.println(time_array[time_index]);
      time_printed_state = time_index;
    }

  }

  // measuring
  if(state == 2) {
    if(measuring == 0) {
      time_step = time_array[time_index] * 1000;
      measuring = 1;

      int chk = DHT11.read(DHT11PIN);
      humidity_result = (float)DHT11.humidity;
      temperature_result = (float)DHT11.temperature;
      
      Serial.println("Measuring...");
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Measuring...");
    }

    int chk = DHT11.read(DHT11PIN);
    humidity_result = (humidity_result + (float)DHT11.humidity) / 2;
    temperature_result = (temperature_result + (float)DHT11.temperature) / 2;

    if(measuring == 1)
      setup_pwm_led();
  }

  if(state == 3) {
    if(print_state != 3) {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Temp (C):");
      lcd.setCursor(9, 0);
      lcd.print(temperature_result);
      lcd.setCursor(0, 1);
      lcd.print("Humidity:");
      lcd.setCursor(10, 1);
      lcd.print(humidity_result);

      Serial.print("Temperature (C): ");
      Serial.println(temperature_result);

      Serial.print("Humidity:");
      Serial.println(humidity_result);

      print_state = 3;

      color_intensity = 0;
      setup_pwm_led();

    }
  }
}
